/*
 * gsampler-channel-dialog.c
 * Copyright (C) Lilian Monchalin 2011 <isaya07@users.sf.net>
 *
 * gsampler is free software: you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * gsampler is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <glib/gi18n-lib.h>

#include "gsampler-channel-dialog.h"
#include "gsampler-utils.h"
#include "gsampler-option.h"
#include "gsampler-instrument.h"

G_DEFINE_TYPE (GsamplerChannelDialog, gsampler_channel_dialog, GTK_TYPE_DIALOG);

static void 
gsampler_channel_dialog_file_selected (GtkFileChooserButton* sender, gpointer data) {
	gchar* filename;
	gchar* ext;
	GsamplerInstrument* instrument;
	GsamplerChannelDialog* self = (GsamplerChannelDialog*) data;
	g_return_if_fail (self != NULL);
	g_return_if_fail (sender != NULL);
	
	filename = gtk_file_chooser_get_filename ((GtkFileChooser*) sender);
	ext = g_utf8_strup ((const gchar*) gsampler_utils_get_extension (filename), (gssize) (-1));
	if (g_strcmp0 (ext, "GIG") == 0) {
		gtk_combo_box_set_active ((GtkComboBox*) self->engine, 0);
	}
	if (g_strcmp0 (ext, "SF2") == 0) {
		gtk_combo_box_set_active ((GtkComboBox*) self->engine, 1);
	}
	if (g_strcmp0 (ext, "SFZ") == 0) {
		gtk_combo_box_set_active ((GtkComboBox*) self->engine, 2);
	}
	gsampler_text_combo_remove_all (self->inst);
	instrument = gsampler_instrument_new ();
	if (gsampler_instrument_load_file (instrument, filename)) {
		GSList* inst_list = NULL;
		
		inst_list = gsampler_instrument_get_list (instrument);
		while (inst_list) {
			struct Struct_list_widget* instru;
			instru = (struct Struct_list_widget*) inst_list->data;
			gsampler_text_combo_insert (self->inst, (const gchar*) instru->Sdata, instru->Sid);
			inst_list = inst_list->next;
		}
		gtk_combo_box_set_active ((GtkComboBox*) self->inst, 0);
	}
	g_free (ext);
	g_free (filename);
}

static void 
gsampler_channel_dialog_init (GsamplerChannelDialog * self) 
{
	GsamplerOption* option;
	GtkTable* table;
	GtkFileFilter* filter;
	gchar* path;
	GtkLabel* engine_label;
	GtkLabel* file_label;
	GtkLabel* inst_label;
	GtkBox* vbox1;

	option = gsampler_option_get_instance ();
	gtk_window_set_title ((GtkWindow*) self, _("Create Channel"));
	gtk_container_set_border_width ((GtkContainer*) self, 5);
	gtk_window_set_default_size ((GtkWindow*) self, 300, 100);
	gtk_dialog_add_button ((GtkDialog*) self, GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL);
	gtk_dialog_add_button ((GtkDialog*) self, GTK_STOCK_APPLY, GTK_RESPONSE_APPLY);
	
	table = (GtkTable*) gtk_table_new (3, 2, TRUE);
	self->engine = gsampler_text_combo_create ();
	self->file = (GtkFileChooserButton*) gtk_file_chooser_button_new (_("Choose a sample file"), 
	                                                                  GTK_FILE_CHOOSER_ACTION_OPEN);
	filter = gtk_file_filter_new ();
	//gtk_buildable_set_name ((GtkBuildable*) filter, "Sample");
	gtk_file_filter_add_pattern (filter, "*.gig");
	gtk_file_filter_add_pattern (filter, "*.GIG");
	gtk_file_filter_add_pattern (filter, "*.sf2");
	gtk_file_filter_add_pattern (filter, "*.SF2");
	gtk_file_filter_add_pattern (filter, "*.SFZ");
	gtk_file_filter_add_pattern (filter, "*.sfz");
	gtk_file_chooser_set_filter ((GtkFileChooser*) self->file, filter);

	self->inst = gsampler_text_combo_create ();
	path = gsampler_option_get_string (option, "Client", "sample-path");

	gtk_file_chooser_set_current_folder ((GtkFileChooser*) self->file, path);
	g_signal_connect_object (self->file, "file-set", 
	                         (GCallback) gsampler_channel_dialog_file_selected, self, 0);
	engine_label = (GtkLabel*) gtk_label_new ("Select Engine :");
	file_label = (GtkLabel*) gtk_label_new ("Select File :");
	inst_label = (GtkLabel*) gtk_label_new ("Select Instrument :");
	gtk_table_attach_defaults (table, (GtkWidget*) file_label, 0, 1, 0, 1);
	gtk_table_attach_defaults (table, (GtkWidget*) self->file, 1, 2, 0, 1);
	gtk_table_attach_defaults (table, (GtkWidget*) inst_label, 0, 1, 1, 2);
	gtk_table_attach_defaults (table, (GtkWidget*) self->inst, 1, 2, 1, 2);
	gtk_table_attach_defaults (table, (GtkWidget*) engine_label, 0, 1, 2, 3);
	gtk_table_attach_defaults (table, (GtkWidget*) self->engine, 1, 2, 2, 3);
	vbox1 = (GtkBox*) gtk_dialog_get_content_area ((GtkDialog*) self);
	gtk_box_pack_start (vbox1, (GtkWidget*) table, FALSE, TRUE, 0);
	gtk_widget_show_all ((GtkWidget*) self);

	g_free (path);
}

static void 
gsampler_channel_dialog_finalize (GObject* obj) 
{
	G_OBJECT_CLASS (gsampler_channel_dialog_parent_class)->finalize (obj);
}

static void 
gsampler_channel_dialog_class_init (GsamplerChannelDialogClass * klass) 
{
	gsampler_channel_dialog_parent_class = g_type_class_peek_parent (klass);
	G_OBJECT_CLASS (klass)->finalize = gsampler_channel_dialog_finalize;
}

GsamplerChannelDialog* 
gsampler_channel_dialog_new (void) 
{
	return g_object_new (GSAMPLER_TYPE_CHANNEL_DIALOG, NULL);
}